<?php

/*$mdocInfo
 Author: Hauke Goos-Habermann (HHabermann@pc-kiel.de)
 Description: Class for FreeIPA access.
$*/

class CFreeIPA extends CChecks
{
	const FREEIPA_CONFIG_CACHE_FILE = '/m23/var/freeIPAConfigCache.php',
		LDAP_TRIM = " \n\r\t\v\0,",
		M23ADMIN = 'm23admin';

	protected
		$defaultDomain = NULL,
		$KRB_realm = NULL,
		$freeIPAServerFQDN = NULL,
		$freeIPAServerIP = NULL,
		$password = NULL,
		$curlHandle = NULL,
		$curlCookieFile = NULL;





/**
**name CFreeIPA::__construct($xmlFile, $argv)
**description Constructor for new CFreeIPA objects.
**parameter serverName: Name of the LDAP server
**/
	public function __construct()
	{
		if (!$this->loadFreeIPAConfigFile())
			throw new Exception($this->popErrorMessagesHTML());

		$this->curlInit();
		$this->curlLogin();
	}





/**
**name CFreeIPA::safeJSONString($in)
**description Makes the input string safe for using it in a JSON query.
**parameter in: Input string.
**returns Safe made string.
**/
	public static function safeJSONString($in)
	{
		return(I18N_safeUTF8_encode(str_replace ('"', '\"', $in)));
	}





/**
**name CFreeIPA::curlInit()
**description Prepares curl to connect to the FreeIPA server.
**/
	private function curlInit()
	{
		$this->curlHandle = curl_init();
		$this->curlCookieFile = uniqid('/m23/tmp/curlCookieFile-');
	}





/**
**name CFreeIPA::curlClose()
**description Closes the connection to the FreeIPA server.
**/
	private function curlClose()
	{
		@unlink($this->curlCookieFile);
		curl_close($this->curlHandle);
	}





/**
**name CFreeIPA::curlExec($post, $json = true)
**description Executes a command on the FreeIPA server via the JSONRPC API.
**parameter post: Data (urlencoded or JSON) to send via POST to the FreeIPA server.
**parameter json: Set to true, if the post data is in JSON format.
**returns Associative array, if JSON is active, otherwise direct output of curl.
**/
	private function curlExec($post, $json = true)
	{
		$ipaServer = $this->getServerFQDN();
	
		// Get select the connection parameter by the POST data format
		if ($json)
		{
			$headers[] = 'Content-Type:application/json';
			$headers[] = 'Accept:applicaton/json';
			$url = "https://$ipaServer/ipa/session/json";
		}
		else
		{
			$headers[] = 'Content-Type:application/x-www-form-urlencoded';
			$headers[] = 'Accept:text/plain';
			$url = "https://$ipaServer/ipa/session/login_password";
		}

		// Open the API URL on the FreeIPA server
		curl_setopt($this->curlHandle, CURLOPT_URL, $url);

		// Set the referer. Needed for security
		curl_setopt($this->curlHandle, CURLOPT_REFERER, "https://$ipaServer/ipa/");

		// Activate to get get the result (JSON or direct output of curl) from the webserver
		curl_setopt($this->curlHandle, CURLOPT_RETURNTRANSFER, true);
	
		// Set send and receive data format in the header
		curl_setopt($this->curlHandle, CURLOPT_HTTPHEADER, $headers);

		// Needs to be set for using the connection after login for more commands, even if the cookie file isn't written
		curl_setopt($this->curlHandle, CURLOPT_COOKIEFILE, $this->curlCookieFile);
		curl_setopt($this->curlHandle, CURLOPT_COOKIEJAR, $this->curlCookieFile);
	
		// Don't check FreeIPA's SSL certificate
		curl_setopt($this->curlHandle, CURLOPT_SSL_VERIFYPEER, false);
	
		// Add post data
		curl_setopt($this->curlHandle, CURLOPT_POST, true);
		curl_setopt($this->curlHandle, CURLOPT_POSTFIELDS, $post);

		// Debug
//	 	curl_setopt($this->curlHandle, CURLOPT_HEADER, true);
//	 	curl_setopt($this->curlHandle, CURLOPT_VERBOSE, true);

		// Curl FreeIPA's JSONRPC and get the result
		$ret = curl_exec($this->curlHandle);

		if ($json)
			// Decode JSON data into an associative array
			return(json_decode($ret, true));
		else
			// Give back (HTML) data directly
			return($ret);
	}





/**
**name CFreeIPA::curlLogin()
**description Logins into the FreeIPA server via the JSONRPC API.
**returns true, if login was successfully, false otherwise.
**/
	private function curlLogin()
	{
		$username = CFreeIPA::M23ADMIN;
		$password = $this->getm23AdminPassword();
		$ret = $this->curlExec("user=$username&password=$password", false);
		
		return($ret == '');
	}





/**
**name CFreeIPA::loadFreeIPAConfigFile()
**description Loads the FreeIPA configuration from CFreeIPA::FREEIPA_CONFIG_CACHE_FILE.
**returns true, if the config file could be read, false otherwise.
**/
	private function loadFreeIPAConfigFile()
	{
		include("/m23/inc/i18n/".$GLOBALS["m23_language"]."/m23base.php");
		
		$temp = array();
	
		// Check, if the config file is present
		if (file_exists(CFreeIPA::FREEIPA_CONFIG_CACHE_FILE))
			$temp = (unserialize(file_get_contents(CFreeIPA::FREEIPA_CONFIG_CACHE_FILE)));
		else
		{
			$this->addErrorMessage($I18N_freeIPAErrorNoConfigFileFound);
			return(false);
		}

		// Read the config file
		$temp = (unserialize(file_get_contents(CFreeIPA::FREEIPA_CONFIG_CACHE_FILE)));

		// Get the needed variables and complain, if they are missing
		foreach (array('defaultDomain', 'KRB_realm', 'freeIPAServerFQDN', 'freeIPAServerIP', 'password') as $var)
		{
			if (!isset($temp[$var]{1}))
				$this->addErrorMessage("$I18N_freeIPAErrorConfigVariableMissing: $var");
			else
				$this->$var = $temp[$var];
		}

		if ($this->hasErrors())
			return(false);
		else
			return(true);
	}





/**
**name CFreeIPA::getDomain()
**description Gets the domain that the FreeIPA server manages.
**returns Domain that the FreeIPA server manages
**/
	public function getDomain()
	{
		return($this->defaultDomain);
	}





/**
**name CFreeIPA::getRealm()
**description Gets the Kerberos realm that the FreeIPA server manages.
**returns Kerberos realm that the FreeIPA server manages
**/
	public function getRealm()
	{
		return($this->KRB_realm);
	}





/**
**name CFreeIPA::getServerFQDN()
**description Gets the FQDN of the FreeIPA server.
**returns FQDN of the FreeIPA server
**/
	public function getServerFQDN()
	{
		return($this->freeIPAServerFQDN);
	}





/**
**name CFreeIPA::getm23AdminPassword()
**description Gets the password of the m23admin user on the FreeIPA server.
**returns Password of the m23admin user on the FreeIPA server.
**/
	public function getm23AdminPassword()
	{
		return($this->password);
	}





/**
**name CFreeIPA::getServerIP()
**description Gets the IP of the FreeIPA server.
**returns IP of the FreeIPA server
**/
	public function getServerIP()
	{
		return($this->freeIPAServerIP);
	}





/**
**name CFreeIPA::deleteConfigCacheFile()
**description Deletes the config file with cached information about the FreeIPA server.
** returns: true, if the config file could be deleted, false otherwise.
**/
	public function deleteConfigCacheFile()
	{
		// Remove the FreeIPA server entry form /etc/hosts
		SERVER_delEtcHosts($this->getServerFQDN());
		return(@unlink(CFreeIPA::FREEIPA_CONFIG_CACHE_FILE));
	}





/**
**name CFreeIPA::generateConfigCacheFileFromFreeIPA($ipaServerIP)
**description Logs into the FreeIPA server via SSH and reads out needed information and stores them in the config file. This requires the user m23admin to be created in FreeIPA with admin right
** returns: true on success, false on errors when storing the config file, string with error message on SSH errors.
**/
	public static function generateConfigCacheFileFromFreeIPA($ipaServerIP)
	{
		$out = array();
		
		$cmds = '# Read domain from IPA database
domain="$(ipa realmdomains-show | grep Domain | head -1 | tr -d \'[:blank:]\' | cut -d\':\' -f2)"

# Read domain from hostname/FQDN (FQDN hostname - domain)
domain2="$(hostname -f | sed "s/$(hostname -a)\.//")"

if [ $domain != $domain2 ]
then
	echo "ERROR: Domain mismatch $domain vs. $domain2!"
	exit 1
fi


# Get Kerberos REALM
realm="$(grep default_realm /etc/krb5.conf | tr -d "[:blank:]" | cut -d"=" -f2)"

if [ -z "$realm" ]
then
	echo "ERROR: No Kerberos REALM given!"
	exit 2
fi

# Read FQDN of the server
fqdn="$(hostname -f)"

if [ -z "$fqdn" ]
then
	echo "ERROR: IPA server has no FQDN!"
	exit 3
fi

if [ "$(echo $fqdn | sed \'s/\./\n/g\' | wc -l)" -eq 0 ]
then
	echo "ERROR: $fqdn is no FQDN!"
	exit 4
fi

# Get IP of the server
ip=$(hostname -I | cut -d\' \' -f1)

if [ $(echo $ip | wc -l) -ne 1 ]
then
	echo "ERROR: $ip is no single IPv4 IP!"
	exit 5
fi

if [ $(echo $ip | sed \'s/\./\n/g\' | wc -l) -ne 4 ]
then
	echo "ERROR: $ip is no IPv4 IP!"
	exit 6
fi

# Show the gathered information
echo "defaultDomain:$domain
KRB_realm:$realm
freeIPAServerFQDN:$fqdn
freeIPAServerIP:$ip"';

		$ret = CLIENT_executeOnClientOrIPSimple($ipaServerIP, 'generateConfigCacheFileFromFreeIPA', $cmds, 'm23admin', '/var/tmp');


		// Check for apparent error messages
		if (strpos($ret, 'Permission denied') != false || strpos($ret, 'Kerberos credentials') != false || strpos($ret, 'ERROR:') != false)
			return($ret);


		// Run thru the lines
		foreach (explode("\n", $ret) as $line)
		{
			if ('' == $line) continue;
			$varVal = explode(':', $line);
			$out[$varVal[0]] = $varVal[1];
		}

		if (count($out) == 4)
		{
			// Add the FQDN with its IP to /etc/hosts to make it resolvable
			SERVER_addEtcHosts($out['freeIPAServerFQDN'], $out['freeIPAServerIP']);
			 
			// Store the config file
			return(CFreeIPA::saveFreeIPAConfigFile($out));
		}
		else
			return(false);
	}





/**
**name CFreeIPA::saveFreeIPAConfigFile($append)
**description Writes/appends entries to the FreeIPA configuration file.
**parameter append: Array with entries to append.
**returns true, if the config file could be written, false otherwise.
**/
	public static function saveFreeIPAConfigFile($append)
	{
		$out = array();

		// Read an existing config file into an array
		if (file_exists(CFreeIPA::FREEIPA_CONFIG_CACHE_FILE))
			$out = (unserialize(file_get_contents(CFreeIPA::FREEIPA_CONFIG_CACHE_FILE)));

		// Append new entries
		$out = array_merge($out, $append);

		// Write all entries
		return(file_put_contents(CFreeIPA::FREEIPA_CONFIG_CACHE_FILE, serialize($out)) !== false);
	}






/**
**name CFreeIPA::__destruct()
**description Destructor for a CFreeIPA object.
**/
	function __destruct()
	{
		$this->curlClose();
		// Show messages before destructing the object
		$this->showMessages();
	}





/**
**name CFreeIPA::getBash_checkRootAndKerberosTicket()
**description Generates BASH code, to check, if the code is run as root and if a valid Kerberos ticket is present.
**returns Generated BASH code
**/
	public static function getBash_checkRootAndKerberosTicket()
	{
		include("/m23/inc/i18n/".$GLOBALS["m23_language"]."/m23base.php");

		return('
export LC_ALL=\'C.UTF-8\'

# '.$I18N_Run_as_root_to_make_chown_work_on_the_administrators_home_directory.'
if [ $(whoami) != \'root\' ]
then
	echo "'.$I18N_ERROR_Not_logged_in_as_root.'"
	exit 1
fi

# '.$I18N_If_there_is_no_ticket_for_admin_pull_one_to_make_ipa_user_add_etc_work.'
if ! klist -s
then
	kinit admin
fi
');
	}





/**
**name CFreeIPA::getBash_addAdminRole($login)
**description Generates BASH code, to assign the admin role to an existing user.
**parameter login: Login name
**returns Generated BASH code
**/
	public static function getBash_addAdminRole($login)
	{
		include("/m23/inc/i18n/".$GLOBALS["m23_language"]."/m23base.php");

		return('
# '.$I18N_Assign_admin_role.'
ipa group-add-member --users="'.$login.'" admins
ipa group-add-member --users="'.$login.'" \'trust admins\'
');
	}





/**
**name CFreeIPA::getBash_getKerberosTicketOnLogin($login, $password)
**description Generates BASH code, to fetch a kerberos ticket on login automatically.
**parameter login: Login name
**returns Generated BASH code
**/
	public static function getBash_getKerberosTicketOnLogin($login, $password)
	{
		include("/m23/inc/i18n/".$GLOBALS["m23_language"]."/m23base.php");

		return('
# '.$I18N_Get_home_directory_of_the_user.'
homeDir="$(ipa -n user-show "'.$login.'" --raw | grep \'homedirectory\' | sed \'s/.*: //\')"
keytab="$homeDir/ipa.keytab"
bashprofile="$homeDir/.bash_profile"

# '.$I18N_Create_home_directory_for_the_user.'
mkdir -p "$homeDir"
chmod 750 "$homeDir"

# '.$I18N_Read_and_save_keytab.'
rm -f "$keytab"
yes "'.$password.'" | ipa-getkeytab -P -p "'.$login.'" -k "$keytab"
chmod 640 "$keytab"

# '.$I18N_Get_kerberos_ticket_on_login.'
if [ ! -f $bashprofile ] || [ $(grep -c $keytab $bashprofile) -eq 0 ]
then
	echo "
export LC_ALL=\'C.UTF-8\'
if ! klist -s
then
	kinit -k -t \'$keytab\' \''.$login.'\'
fi
" >> $bashprofile

fi

# '.$I18N_Restart_SSS_and_SSH_to_make_UID_and_GID_available_in_FreeIPA.'
service sssd restart
service sshd restart

sleep 5

# '.$I18N_Read_UID_and_GID_from_FreeIPA.'
uid=$(ipa -n user-show "'.$login.'" --raw | grep \'uidnumber:\' | tr -d \'[:blank:]\' | cut -d\':\' -f2)
gid=$(ipa -n user-show "'.$login.'" --raw | grep \'gidnumber:\' | tr -d \'[:blank:]\' | cut -d\':\' -f2)

# '.$I18N_Set_UID_and_GID_on_the_home_directory_of_the_user.'
chown -R $uid.$gid "$homeDir"
');
	}





/**
**name CFreeIPA::getBash_createUser($login, $firstName, $lastName, $password, $shell = '/bin/bash', $sshPubKey = NULL)
**description Generates BASH code, to create a user in FreeIPA.
**parameter login: Login name
**parameter firstName: Forename
**parameter lastName: Familyname
**parameter password: Password
**parameter shell: Shell to use when logged in
**parameter sshPubKey: Public SSH key to allow passwordless login
**returns Generated BASH code
**/
	public static function getBash_createUser($login, $firstName, $lastName, $password, $shell = '/bin/bash', $sshPubKey = NULL)
	{
		$date = '2050-01-01Z';
	
		include("/m23/inc/i18n/".$GLOBALS["m23_language"]."/m23base.php");

		// Generate parameter for adding the SSH pubkey, if given
		$sshPubKeyParam = is_null($sshPubKey) ? '' : '--sshpubkey="'.$sshPubKey.'"';

		return('
# '.$I18N_Create_user_in_FreeIPA.'
yes "'.$password.'" | ipa user-add "'.$login.'" '.$sshPubKeyParam.' --first="'.$firstName.'" --last="'.$lastName.'" --shell="'.$shell.'"  --password

ipa user-mod "'.$login.'" --user-auth-type=password

');
	}





/**
**name CFreeIPA::getBash_deleteUser($login)
**description Generates BASH code, to delete a user from FreeIPA.
**parameter login: Login name
**returns Generated BASH code
**/
	public static function getBash_deleteUser($login)
	{
		include("/m23/inc/i18n/".$GLOBALS["m23_language"]."/m23base.php");

		return('
		# Get home directory of the user
		homeDir="$(ipa -n user-show "'.$login.'" --raw | grep "homedirectory" | sed "s/.*: //")"

		# Delete the user, if it exists
		if [ "$homeDir" ]
		then
			# Delete the user account
			ipa user-del "'.$login.'" --no-preserve
			
			# Delete the home directory
			rm -r -f "$homeDir"
		fi
		');
	}





/**
**name CFreeIPA::getBash_setPasswordExpiration($login, $password, $date = '2050-01-01Z')
**description Generates BASH code, to set expiration date for the password (far into the future so that you are not prompted to change the password).
**parameter login: Login name
**parameter password: Password
**parameter date: Expiration date.
**returns Generated BASH code
**/
	public static function getBash_setPasswordExpiration($login, $password, $date = '2050-01-01Z')
	{
		include("/m23/inc/i18n/".$GLOBALS["m23_language"]."/m23base.php");

		return('
yes "'.$password.'" | ipa user-mod "'.$login.'" --password

# '.$I18N_Set_password_expiration_date_far_in_the_future.'
ipa user-mod "'.$login.'" --password-expiration="'.$date.'"
');
	}




/**
**name CFreeIPA::getFreeIPIntegrateCommands()
**description Generates commands to create an administrator account, that can be used by m23, on the FreeIPA server.
**returns BASH commands to create an administrator account, that can be used by m23, on the FreeIPA server.
**/
	public static function getFreeIPIntegrateCommands()
	{

		$login = CFreeIPA::M23ADMIN;
		$firstName = 'm23';
		$lastName = 'Admin';
		$password = HELPER_generateSalt(64);
		$shell = '/bin/bash';
		$sshPubKey = SERVER_getPublicSSHKey();

		CFreeIPA::saveFreeIPAConfigFile(array('password' => $password));

		return(
			CFreeIPA::getBash_checkRootAndKerberosTicket().
			CFreeIPA::getBash_deleteUser($login).
			CFreeIPA::getBash_createUser($login, $firstName, $lastName, $password, $shell, $sshPubKey).
			CFreeIPA::getBash_addAdminRole($login).
			CFreeIPA::getBash_setPasswordExpiration($login, $password).
			CFreeIPA::getBash_getKerberosTicketOnLogin($login, $password)
			);
	}





/**
**name CFreeIPA::GUI_showConfigureServer()
**description Shows a dialog to integrate an existing FreeIPA server into m23.
**/
	public static function GUI_showConfigureServer()
	{
		include("/m23/inc/i18n/".$GLOBALS["m23_language"]."/m23base.php");
// 		HTML_logArea('LA_freeIPIntegrateCommands', 100, 25, CFreeIPA::getFreeIPIntegrateCommands());
		
		// Commands for integrating a FreeIPA server into m23
		$freeIPAWgetCommands = "wget http://".getServerIP()."/getFreeIPIntegrateCommands.php?lang=$GLOBALS[m23_language] -O /tmp/FreeIPIntegrate.sh\nbash /tmp/FreeIPIntegrate.sh";
		HTML_logArea('LA_freeIPAWgetCommands', 100, 2, $freeIPAWgetCommands);

		// IP of the FreeIPA server
		$ip = HTML_input('ED_ip', false, 17, 17);

		// Button to connect to the FreeIPA server and store settings
		if (HTML_submit("BUT_checkConnectionAndSaveSettingsOnSuccess", $I18N_checkConnectionAndSaveSettingsOnSuccess))
		{
			if (checkIP($ip))
			{
				$ret = CFreeIPA::generateConfigCacheFileFromFreeIPA($ip);
				
				if ($ret === true)
					MSG_showInfo($I18N_connectionTestWithIPAServerWasSuccessfully);
				elseif ($ret === false)
					MSG_showError($I18N_errorWhileTryingToConnectToTheFreeIPAServer);
				else
					MSG_showError($ret);
			}
			else
				MSG_showError($I18N_invalid_ip);
		}

		// Button to disconnect from the FreeIPA server
		if (HTML_submit("BUT_disconnectfromFreeIPAserver", $I18N_disconnectfromFreeIPAserver))
		{
			$CFreeIPAO = CFreeIPA::createCFreeIPAObject($createErrorMsg);
			if ($CFreeIPAO === false)
				MSG_showError($I18N_configFileForConnectingTheFreeIPAserverCouldNotBeDeleted);
			else
			{
				if ($CFreeIPAO->deleteConfigCacheFile())
					MSG_showInfo($I18N_configFileForConnectingTheFreeIPAserverWasDeleted);
				else
					MSG_showError($I18N_configFileForConnectingTheFreeIPAserverCouldNotBeDeleted);
			}
		}

		// Try to create a CFreeIPA object. This only works if there is a configuration file on the m23 server
		$CFreeIPAO = CFreeIPA::createCFreeIPAObject($createErrorMsg);
		if ($CFreeIPAO === false)
			MSG_showWarning($createErrorMsg);

		// FreeIPA server was integrated (config file present)
		if ($CFreeIPAO !== false)
		{
			HTML_showTitle($I18N_settings);
			HTML_showTableHeader(true, 'subtable2', 'width="100%" cellspacing=10');
			HTML_showTableRow($I18N_FreeIPAServerHostname, $CFreeIPAO->getServerFQDN());
			HTML_showTableRow($I18N_FreeIPAServerIP, $CFreeIPAO->getServerIP());
			HTML_showTableRow($I18N_domain, $CFreeIPAO->getDomain());
			HTML_showTableRow($I18N_KerberosRealm, $CFreeIPAO->getRealm());
			HTML_showTableRow($I18N_status, $CFreeIPAO->isOnline() ? $I18N_computerOn : $I18N_computerOff);
			HTML_showTableRow(array('td' => 'colspan="2"'), '<center>'.BUT_disconnectfromFreeIPAserver.'</center>');
			
			HTML_showTableEnd(true);
		}
		// FreeIPA server was NOT integrated (no config file present)
		else
		{
			HTML_showTitle($I18N_connectWithFreeIPAServer);
			HTML_showTableHeader(true, 'subtable2', 'width="100%" cellspacing=10');
			echo("
			<table align=\"center\">
			<tr>
				<td><div class=\"subtable_shadow\">
					<table align=\"center\" class=\"subtable2\">
						<tr>
							<td class=\"numberBorder\">1.</td>
							<td>
								".LA_freeIPAWgetCommands."
							<td> $I18N_help_freeIPAIntegrateCommands</td>
						</tr>
						<tr>
							<td class=\"numberBorder\">2.</td>
							<td>
								$I18N_ipAddressOfFreeIPAServer: ".ED_ip."</br></br>
								".BUT_checkConnectionAndSaveSettingsOnSuccess."
							</td>
							<td>$I18N_help_checkConnectionToFreeIPA</td>
						</tr>
					</table>
				</div></td>
			<tr></table>
			");
		}

	}





/**
**name CFreeIPA::isAvailable()
**description Checks, if a FreeIPA server is integrated into m23, by checking the presence ofthe configuration file on the m23 server. A presence doesn't mean, that the FreeIPA server is currently running.
**returns true, if the config file is present, otherwise false.
**/
	public static function isAvailable()
	{
		return(file_exists(CFreeIPA::FREEIPA_CONFIG_CACHE_FILE));
	}





/**
**name CFreeIPA::createCFreeIPAObject(&$errorMsg)
**description Tries to create a CFreeIPA object. This only works if there is a configuration file on the m23 server.
**parameter errorMsg: In case of an error the error message is written into this variable.
**returns CFreeIPA object on success, otherwise false.
**/
	public static function createCFreeIPAObject(&$errorMsg)
	{
		try
		{
			$CFreeIPAO = new CFreeIPA();
		}
		catch (Exception $e)
		{
			$errorMsg = $e->getMessage();
		}

		return(isset($CFreeIPAO) && is_object($CFreeIPAO) ? $CFreeIPAO : false);
	}





/**
**name CFreeIPA::executeOnFreeIPA($jobName, $cmds)
**description Runs a BASH script on the FreeIPA server.
**parameter jobName: name of the job screen should show
**parameter cmds: the commands of the script
**returns Output of the script.
**/
	private function executeOnFreeIPA($jobName, $cmds)
	{
		return(CLIENT_executeOnClientOrIPSimple($this->getServerIP(), $jobName, $cmds, 'm23admin', '/var/tmp'));
	}





/**
**name CFreeIPA::isOnline()
**description Checks, if the FreeIPA server, that was integrated into m23, is currently running.
**returns true, if the FreeIPA server is running, false otherwise.
**/
	private function isOnline()
	{
		$ret = $this->curlExec(
		'{
			"id": 0,
			"method": "user_find",
			"params": [
				[
					"'.CFreeIPA::M23ADMIN.'"
				],
				{
				}
			]
		}');
		return(strpos($ret['result']['summary'], '1 user matched') === 0);
// 		return(strpos($this->executeOnFreeIPA('isOnline', 'ipa user-find "'.CFreeIPA::M23ADMIN.'"'),'m23admin') !== false);
	}





/**
**name CFreeIPA::getUsedFreeIPAIPs($intIPs = false)
**description Gets the IPs that are managed by the FreeIPA server.
**parameter intIPs: Set to true, to return the IPs as int values.
**returns IPs that are managed by the FreeIPA server.
**/
	public function getUsedFreeIPAIPs($intIPs = false)
	{
		$out = array();

		$ret = $this->curlExec(
		'{
			"id": 0,
			"method": "dnsrecord_find",
			"params": [
				[
					"'.$this->getDomain().'"
				],
				{
				}
			]
		}');

		// Give out an empty array, if there is no (valid) answer from the FreeIPA server
		if (!is_array($ret))
			return(array());

		foreach ($ret['result']['result'] as $entry)
		{
			if (isset($entry['arecord'][0]))
			{
				if ($intIPs)
					$out[$entry['arecord'][0]] = ip2longSafe($entry['arecord'][0]);
				else
					$out[$entry['arecord'][0]] = $entry['arecord'][0];
			}
		}

		return($out);

/*
		$cmds = "ipa dnsrecord-find '".$this->getDomain()."' --raw | grep 'arecord:' | tr -d '[:blank:]' | cut -d':' -f2 | sort -u";
		$ret = $this->executeOnFreeIPA('getUsedFreeIPAIPs', $cmds);

		// Is there anything given?
		if (isset($ret{1}))
			// Return all finds (this includes the IP of the FreeIPA server too)
			$out = array_filter(explode("\n",$ret));
		else
			// Nothing returned by the command??? => Return the IP of the FreeIPA server
			$out = array($this->getServerIP());

		// Convert the IPs to Int or not
		return($intIPs ? array_map('ip2longSafe', $out) : $out);
*/
	}





/**
**name CFreeIPA::checkCurlResult($ret)
**description Checks the answer of a FreeIPA curl call.
**parameter ret: Answer of a FreeIPA curl call.
**returns true, if there are no errors in the query, otherwise false.
**/
	private function checkCurlResult($ret, $ignoreError = 'EiJ5Toowephei2vohXovojo')
	{
		if (isset($ret['error']['message']{1}))
		{
			if (strpos($ret['error']['message'], $ignoreError)!==false)
				return(true);
		
			$this->addErrorMessage($ret['error']['message']);
			return(false);
		}

		if (is_array($ret['error']))
		{
			$error = print_r($ret['error'], true);

			if (strpos($error, $ignoreError)!==false)
				return(true);

			$this->addErrorMessage($error);
			return(false);
		}

		return(true);
	}




	public function addDNSEntry($client, $clientIP)
	{
		$domain = $this->getDomain();

		// Add DNS record
		$ret = $this->curlExec(
		'{
			"id": 0,
			"method": "dnsrecord_add",
			"params": [
				[
					 "'.$domain.'",
					{
						"__dns_name__": "'.$client.'"
					}
				],
				{
					"arecord":
					[
						"'.$clientIP.'"
					]
				}
			]
		}');
		
		return($this->checkCurlResult($ret));
	}




/**
**name CFreeIPA::addComputer($client, $clientIP, $mac, $description = '', $class = '', $locality = '', $location = '', $platform = '', $os = '')
**description Adds an m23 client to FreeIPA as host and into FreeIPA's DNS.
**parameter client: Name of the computer/client to add.
**parameter clientIP: The IP of the client.
**parameter mac: MAC address of the client.
**parameter description: Description to store in FreeIPA.
**parameter class: Host category
**parameter locality: Locality where the computer is placed.
**parameter location: Location where the computer is placed.
**parameter platform: Platform (e.g. CPU, computer model, VM, etc.)
**parameter os: The distribution, OS that the computer runs.
**returns true, if the client was added successfully, otherwise (error) messages from the FreeIPA API.
**/
	public function addComputer($client, $clientIP, $mac, $description = '', $class = '', $locality = '', $location = '', $platform = '', $os = '')
	{
		$params = '';
	
		$domain = $this->getDomain();

		$mac = strtoupper(CLIENT_convertMac($mac, ':'));

		$ret = $this->addDNSEntry($client, $clientIP);
		// Return error message in case of an error
		if ($ret !== true) return($ret);

		// Add host
		$ret = $this->curlExec('
		{
			"id": 0,
			"method": "host_add",
			"params": [
				[
					"'.$client.'.'.$domain.'"
				],
				{
					"description": "'.CFreeIPA::safeJSONString($description).'",
					"l": "'.CFreeIPA::safeJSONString($locality).'",
					"nshardwareplatform": "'.CFreeIPA::safeJSONString($platform).'",
					"nshostlocation": "'.CFreeIPA::safeJSONString($location).'",
					"macaddress": "'.$mac.'",
					"userclass": "'.CFreeIPA::safeJSONString($class).'",
					"nsosversion": "'.CFreeIPA::safeJSONString($os).'"
				}
			]
		}');
		

		// Return error message in case of an error
		return($this->checkCurlResult($ret));

/*
		if (!is_null($description)) $params .= " --desc='$description'";
		if (!is_null($locality)) $params .= " --locality='$locality'";
		if (!is_null($location)) $params .= " --location='$location'";
		if (!is_null($platform)) $params .= " --platform='$platform'";
		if (!is_null($os)) $params .= " --os='$os'";

		$cmds = "
		export LC_ALL='C.UTF-8'

		ipa dnsrecord-add '$domain' '$client' --a-rec '$clientIP'
		ret1=$?
		ipa host-add '$client.$domain' $params
		ret2=$?
		if [ \$ret1 -eq 0 ] && [ \$ret2 -eq 0 ]
		then
			echo ALL_OK
		fi
		";

		$ret = $this->executeOnFreeIPA('addComputer', $cmds);

		if (strpos($ret, 'ALL_OK')!==false)
			return(true);
		else
			return($ret);
*/
	}





/**
**name CFreeIPA::setComputerBulkPassword($client, $bulkPasswd)
**description Sets the bulk password of an m23 client in FreeIPA.
**parameter client: Name of the computer/client to add.
**parameter bulkPasswd: Bulk password for running ipa-client-install on the client.
**returns true, if the bulk password was set successfully, otherwise (error) messages from the FreeIPA API.
**/
	public function setComputerBulkPassword($client, $bulkPasswd)
	{
		$domain = $this->getDomain();
		
		
		$ret = $this->curlExec(
		'{
			"id": 0,
			"method": "host_disable",
			"params": [
				[
					"'.$client.'.'.$domain.'"
				],
				{
				}
			]
		}
		');
		// Return error message in case of an error
		$ret = $this->checkCurlResult($ret,'already disabled');
		if ($ret !== true) return($ret);

		// After disabling the host, the A record needs to be added again.
		$clientIP = CLIENT_getIPbyName($client);
		$this->addDNSEntry($client, $clientIP);

		$ret = $this->curlExec(
		'{
			"id": 0,
			"method": "host_mod",
			"params": [
				[
					"'.$client.'.'.$domain.'"
				],
				{
					"userpassword": "'.$bulkPasswd.'"
				}
			]
		}');
		
		// Return error message in case of an error
		return($this->checkCurlResult($ret, 'enrolled Xhost'));
		
/*		$cmds = "
		export LC_ALL='C.UTF-8'

		ipa host-mod '$client.$domain' --password='$bulkPasswd'
		if [ \$? -eq 0 ]
		then
			echo ALL_OK
		fi
		";

		$ret = $this->executeOnFreeIPA('setComputerBulkPassword', $cmds);

		if (strpos($ret, 'ALL_OK')!==false)
			return(true);
		else
			return($ret);
*/
	}





/**
**name CFreeIPA::addUser($login, $firstName, $lastName, $password, $shell = '/bin/bash', $sshKey = '')
**description Creates a user in FreeIPA.
**parameter login: Login name
**parameter firstName: Forename
**parameter lastName: Familyname
**parameter password: Password
**parameter shell: Shell to use when logged in
**parameter sshPubKey: Public SSH key to allow passwordless login
**returns true, if the user was added successfully, otherwise (error) messages from the FreeIPA API.
**/
	public function addUser($login, $firstName, $lastName, $password, $shell = '/bin/bash', $sshKey = '')
	{
		$ret = $this->curlExec(
		'{
			"id": 0,
			"method": "user_add",
			"params": [
				[
					"'.$login.'"
				],
				{
					"givenname": "'.CFreeIPA::safeJSONString($firstName).'",
					"ipasshpubkey": [
						"'.$sshKey.'"
					],
					"ipauserauthtype": [
						"password"
					],
					"krbpasswordexpiration": {
						"__datetime__": "20500101000000Z"
					},
					"loginshell": "'.$shell.'",
					"sn": "'.CFreeIPA::safeJSONString($lastName).'",
					"userpassword": "'.$password.'"
				}
			]
		}');

		// Return error message in case of an error
		return($this->checkCurlResult($ret));

/*
		// Fetch commands to create a user and add checking of the return value
		$cmds =
		$this->getBash_createUser($login, $firstName, $lastName, $password)."\nret1=$?\n".
		$this->getBash_setPasswordExpiration($login)."\nret2=$?\n

		if [ \$ret1 -eq 0 ] && [ \$ret2 -eq 0 ]
		then
			echo ALL_OK
		fi
		";

		$ret = $this->executeOnFreeIPA('addUser', $cmds);

		if (strpos($ret, 'ALL_OK')!==false)
			return(true);
		else
			return($ret);
*/
	}





/**
**name CFreeIPA::delUser($login)
**description Deletes a user from FreeIPA.
**parameter login: Login name
**returns true, if the user was deleted successfully, otherwise (error) messages from the ipa tool.
**/
	public function delUser($login)
	{
		$ret = $this->curlExec(
		'{
			"id": 0,
			"method": "user_del",
			"params": [
				[
					[
						"'.$login.'"
					]
				],
				{
					"preserve": false
				}
			]
		}');

		// Return error message in case of an error
		return($this->checkCurlResult($ret));
	
/*
		$cmds = "
		ipa user-del '$login' --no-preserve
		
		if [ \$? -eq 0 ]
		then
			echo ALL_OK
		fi";

		$ret = $this->executeOnFreeIPA('delUser', $cmds);

		if (strpos($ret, 'ALL_OK')!==false) return(true); else return($ret);
	
		return(true);
*/
	}





/**
**name CFreeIPA::delComputer($client)
**description Deletes an m23 client form FreeIPA's host and DNS entries.
**parameter client: Name of the computer/client to add.
**returns true, if the client was deleted successfully, otherwise (error) messages from the ipa tool.
**/
	public function delComputer($client)
	{
		$domain = $this->getDomain();

		// Remove DNS entry
		$ret = $this->curlExec(
		'{
			"id": 0,
			"method": "dnsrecord_del",
			"params": [
				[
					"'.$domain.'",
					{
						"__dns_name__": "'.$client.'"
					}
				],
				{
					"del_all": true
				}
			]
		}');

		// Return error message in case of an error
		$ret = $this->checkCurlResult($ret);
		if ($ret !== true) return($ret);

		// Remove host entry
		$ret = $this->curlExec('
		{
			"id": 0,
			"method": "host_del",
			"params": [
				[
					[
						"'.$client.'.'.$domain.'"
					]
				],
				{
				}
			]
		}');

		// Return error message in case of an error
		return($this->checkCurlResult($ret));

/*
		$cmds = "
		export LC_ALL='C.UTF-8'

		ipa dnsrecord-del --del-all '$domain' '$client'
		ret1=$?
		ipa host-del '$client.$domain'
		ret2=$?
		if [ \$ret1 -eq 0 ] && [ \$ret2 -eq 0 ]
		then
			echo ALL_OK
		fi
		";

		$ret = $this->executeOnFreeIPA('delComputer', $cmds);

		if (strpos($ret, 'ALL_OK')!==false) return(true); else return($ret);
*/
	}





/**
**name CFreeIPA::existUserGroup($group)
**description Checks, if a group is existing in FreeIPA.
**parameter group: Name of the group to check.
**returns true, if the group exists, otherwise false.
**/
	public function existUserGroup($group)
	{
		$ret = $this->curlExec('
		{
			"id": 0,
			"method": "group_show",
			"params": [
				[
					"'.$group.'"
				],
				{
				}
			]
		}
		');

		// Group names are case insensitive
		return(strtoupper($ret['result']['value']) === strtoupper($group));

/*
		$cmds = "
		export LC_ALL='C.UTF-8'

		ipa group-show '$group'
		if [ \$? -eq 0 ]
		then
			echo ALL_OK
		fi
		";

		$ret = $this->executeOnFreeIPA('existUserGroup', $cmds);

		if (strpos($ret, 'ALL_OK')!==false) return(true); else return(false);
*/
	}





/**
**name CFreeIPA::createUserGroup($group, $description)
**description Creates a group in FreeIPA.
**parameter group: Name of the group
**parameter description: description of the group
**returns true, if the group was created, otherwise false.
**/
	public function createUserGroup($group, $description)
	{
		// Don't create a group that already exists
		if ($this->existUserGroup($group)) return(true);

		$ret = $this->curlExec('
		{
			"id": 0,
			"method": "group_add",
			"params": [
				[
					"'.$group.'"
				],
				{
					"description": "'.CFreeIPA::safeJSONString($description).'"
				}
			]
		}
		');

		// Return error message in case of an error
		return($this->checkCurlResult($ret));
/*
		$cmds = "
		export LC_ALL='C.UTF-8'

		ipa group-add --desc='$description' '$group'
		if [ \$? -eq 0 ]
		then
			echo ALL_OK
		fi
		";

		$ret = $this->executeOnFreeIPA('createUserGroup', $cmds);

		if (strpos($ret, 'ALL_OK')!==false) return(true); else return(false);
*/
	}





/**
**name CFreeIPA::addLoginToGroup($login, $group)
**description Adds an user to a group in FreeIPA.
**parameter group: Name of the group
**parameter login: Login name
**returns true, if the login was added or is in the group already, otherwise false.
**/
	public function addLoginToGroup($login, $group)
	{
	
		$ret = $this->curlExec('
		{
			"id": 0,
			"method": "group_add_member",
			"params": [
				[
					"'.$group.'"
				],
				{
					"user": [
						"'.$login.'"
					]
				}
			]
		}');

		// Return error message in case of an error
		return($this->checkCurlResult($ret));
}





/**
**name CFreeIPA::addGroupToUserGroup($group, $mainGroup)
**description Adds a group to another group in FreeIPA.
**parameter group: Name of the group to add to another group
**parameter mainGroup: Name of the main group where the another group should be added
**returns true, if the group was added or is in the group already, otherwise false.
**/
	public function addGroupToUserGroup($group, $mainGroup)
	{
	
	print("<h3>addGroupToUserGroup($group, $mainGroup)</h3>");
	
		$ret = $this->curlExec('
		{
			"id": 0,
			"method": "group_add_member",
			"params": [
				[
					"'.$mainGroup.'"
				],
				{
					"group": [
						"'.$group.'"
					]
				}
			]
		}');

		// Return error message in case of an error
		return($this->checkCurlResult($ret));
	}





/**
**name CFreeIPA::createHostGroup($group, $description)
**description Creates a host (m23 client) group in FreeIPA.
**parameter group: Name of the group
**parameter description: description of the group
**returns true, if the group was created, otherwise false.
**/
	public function createHostGroup($group, $description)
	{
print("<h4>createHostGroup($group, $description)</h4>");
		// Don't create a group that already exists
		if ($this->existHostGroup($group)) return(true);
print("<h4>createHostGroup($group, $description): weiter</h4>");

		$ret = $this->curlExec('
		{
			"id": 0,
			"method": "hostgroup_add",
			"params": [
				[
					"'.$group.'"
				],
				{
					"description": "'.CFreeIPA::safeJSONString($description).'"
				}
			]
		}
		');

		// Return error message in case of an error
		return($this->checkCurlResult($ret));
	}





/**
**name CFreeIPA::existHostGroup($group)
**description Checks, if a host group is existing in FreeIPA.
**parameter group: Name of the group to check.
**returns true, if the group exists, otherwise false.
**/
	public function existHostGroup($group)
	{
		$ret = $this->curlExec('
		{
			"id": 0,
			"method": "hostgroup_show",
			"params": [
				[
					"'.$group.'"
				],
				{
				}
			]
		}
		');

		// Group names are case insensitive
		return(strtoupper($ret['result']['value']) === strtoupper($group));
	}





/**
**name CFreeIPA::addClientToHostGroup($client, $group)
**description Adds an m23 client to a host group in FreeIPA.
**parameter group: Name of the group
**returns true, if the client was added or is in the group already, otherwise false.
**/
	public function addClientToHostGroup($client, $group)
	{
		$domain = $this->getDomain();
	
		$ret = $this->curlExec('
		{
			"id": 0,
			"method": "hostgroup_add_member",
			"params": [
				[
					"'.$group.'"
				],
				{
					"host": [
						"'.$client.'.'.$domain.'"
					]
				}
			]
		}
		');

		// Return error message in case of an error
		return($this->checkCurlResult($ret));
	}





/**
**name CFreeIPA::addGroupToHostGroup($group, $mainGroup)
**description Adds a host group to another host group in FreeIPA.
**parameter group: Name of the host group to add to another host group
**parameter mainGroup: Name of the main host group where the another host group should be added
**returns true, if the host group was added or is in the host group already, otherwise false.
**/
	public function addGroupToHostGroup($group, $mainGroup)
	{
	
// 	print("<h3>addGroupToUserGroup($group, $mainGroup)</h3>");
	
		$ret = $this->curlExec('
		{
			"id": 0,
			"method": "hostgroup_add_member",
			"params": [
				[
					"'.$group.'"
				],
				{
					"hostgroup": [
						"'.$mainGroup.'"
					]
				}
			]
		}');

		// Return error message in case of an error
		return($this->checkCurlResult($ret));
	}}
?>